@@ -2,7 +2,6 @@
 
 import abc
 import csv
-import json
 import re
 from dataclasses import dataclass, field
 from typing import Any, Callable, Dict, Generator, Iterable, List, Literal, Optional
@@ -113,20 +112,31 @@ def create_csv_rows(data: Iterable[Dict]) -> Generator[str, None, None]:
 
 def create_html_response(
     request: Request,
-    data: str,
+    data: Any,
     templates: Jinja2Templates,
     template_name: str,
+    title: Optional[str] = None,
     router_prefix: Optional[str] = None,
+    **kwargs: Any,
 ) -> _TemplateResponse:
     """Create Template response."""
     urlpath = request.url.path
     if root_path := request.app.root_path:
         urlpath = re.sub(r"^" + root_path, "", urlpath)
 
+    if router_prefix:
+        urlpath = re.sub(r"^" + router_prefix, "", urlpath)
+
     crumbs = []
     baseurl = str(request.base_url).rstrip("/")
 
+    if router_prefix:
+        baseurl += router_prefix
+
     crumbpath = str(baseurl)
+    if urlpath == "/":
+        urlpath = ""
+
     for crumb in urlpath.split("/"):
         crumbpath = crumbpath.rstrip("/")
         part = crumb
@@ -135,24 +145,20 @@ def create_html_response(
         crumbpath += f"/{crumb}"
         crumbs.append({"url": crumbpath.rstrip("/"), "part": part.capitalize()})
 
-    if router_prefix:
-        baseurl += router_prefix
-
     return templates.TemplateResponse(
         request,
         name=f"{template_name}.html",
         context={
-            "response": orjson.loads(data),
+            "response": data,
             "template": {
                 "api_root": baseurl,
                 "params": request.query_params,
-                "title": "",
+                "title": title or template_name,
             },
             "crumbs": crumbs,
-            "url": str(request.url),
-            "baseurl": baseurl,
-            "urlpath": str(request.url.path),
-            "urlparams": str(request.url.query),
+            "url": baseurl + urlpath,
+            "params": str(request.url.query),
+            **kwargs,
         },
     )
 
@@ -181,10 +187,10 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
 
     def __post_init__(self):
         """Post Init: register route and configure specific options."""
-        self.register_routes()
         if self.with_common:
-            self._conformance_route()
             self._landing_route()
+            self._conformance_route()
+        self.register_routes()
 
     def url_for(self, request: Request, name: str, **path_params: Any) -> str:
         """Return full url (with prefix) for a specific handler."""
@@ -207,15 +213,19 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
     def _create_html_response(
         self,
         request: Request,
-        data: str,
+        data: Any,
         template_name: str,
+        title: Optional[str] = None,
+        **kwargs: Any,
     ) -> _TemplateResponse:
         return create_html_response(
             request,
             data,
             templates=self.templates,
             template_name=template_name,
+            title=title,
             router_prefix=self.router_prefix,
+            **kwargs,
         )
 
     @abc.abstractmethod
@@ -262,7 +272,7 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="conformance",
                 )
 
@@ -325,8 +335,9 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="landing",
+                    title=self.title,
                 )
 
             return data
@@ -354,7 +365,7 @@ class OGCFeaturesFactory(EndpointsFactory):
                 rel="data",
             ),
             model.Link(
-                title="Collection metadata",
+                title="Collection metadata (Template URL)",
                 href=self.url_for(
                     request,
                     "collection",
@@ -362,9 +373,10 @@ class OGCFeaturesFactory(EndpointsFactory):
                 ),
                 type=MediaType.json,
                 rel="data",
+                templated=True,
             ),
             model.Link(
-                title="Collection queryables",
+                title="Collection queryables (Template URL)",
                 href=self.url_for(
                     request,
                     "queryables",
@@ -372,15 +384,17 @@ class OGCFeaturesFactory(EndpointsFactory):
                 ),
                 type=MediaType.schemajson,
                 rel="queryables",
+                templated=True,
             ),
             model.Link(
-                title="Collection Features",
+                title="Collection Features (Template URL)",
                 href=self.url_for(request, "items", collectionId="{collectionId}"),
                 type=MediaType.geojson,
                 rel="data",
+                templated=True,
             ),
             model.Link(
-                title="Collection Feature",
+                title="Collection Feature (Template URL)",
                 href=self.url_for(
                     request,
                     "item",
@@ -389,6 +403,7 @@ class OGCFeaturesFactory(EndpointsFactory):
                 ),
                 type=MediaType.geojson,
                 rel="data",
+                templated=True,
             ),
         ]
 
@@ -515,8 +530,9 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="collections",
+                    title="Collections list",
                 )
 
             return data
@@ -602,8 +618,9 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="collection",
+                    title=f"{collection.id} collection",
                 )
 
             return data
@@ -647,8 +664,9 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="queryables",
+                    title=f"{collection.id} queryables",
                 )
 
             return data
@@ -902,7 +920,10 @@ class OGCFeaturesFactory(EndpointsFactory):
             # HTML Response
             if output_type == MediaType.html:
                 return self._create_html_response(
-                    request, orjsonDumps(data).decode(), template_name="items"
+                    request,
+                    orjson.loads(orjsonDumps(data).decode()),
+                    template_name="items",
+                    title=f"{collection.id} items",
                 )
 
             # GeoJSONSeq Response
@@ -1067,8 +1088,9 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    orjsonDumps(data).decode(),
+                    orjson.loads(orjsonDumps(data).decode()),
                     template_name="item",
+                    title=f"{collection.id}/{itemId} item",
                 )
 
             # Default to GeoJSON Response
@@ -1089,9 +1111,9 @@ class OGCTilesFactory(EndpointsFactory):
 
     def links(self, request: Request) -> List[model.Link]:
         """OGC Tiles API links."""
-        return [
+        links = [
             model.Link(
-                title="Collection Vector Tiles",
+                title="Collection Vector Tiles (Template URL)",
                 href=self.url_for(
                     request,
                     "collection_get_tile",
@@ -1102,9 +1124,10 @@ class OGCTilesFactory(EndpointsFactory):
                 ),
                 type=MediaType.mvt,
                 rel="data",
+                templated=True,
             ),
             model.Link(
-                title="Collection TileSets",
+                title="Collection TileSets (Template URL)",
                 href=self.url_for(
                     request,
                     "collection_tileset_list",
@@ -1112,9 +1135,10 @@ class OGCTilesFactory(EndpointsFactory):
                 ),
                 type=MediaType.json,
                 rel="data",
+                templated=True,
             ),
             model.Link(
-                title="Collection TileSet",
+                title="Collection TileSet (Template URL)",
                 href=self.url_for(
                     request,
                     "collection_tileset",
@@ -1123,7 +1147,27 @@ class OGCTilesFactory(EndpointsFactory):
                 ),
                 type=MediaType.json,
                 rel="data",
+                templated=True,
             ),
+        ]
+
+        if self.with_viewer:
+            links.append(
+                model.Link(
+                    title="Collection Map viewer (Template URL)",
+                    href=self.url_for(
+                        request,
+                        "viewer_endpoint",
+                        collectionId="{collectionId}",
+                        tileMatrixSetId="{tileMatrixSetId}",
+                    ),
+                    type=MediaType.html,
+                    rel="data",
+                    templated=True,
+                )
+            )
+
+        links += [
             model.Link(
                 title="TileMatrixSets",
                 href=self.url_for(
@@ -1134,7 +1178,7 @@ class OGCTilesFactory(EndpointsFactory):
                 rel="data",
             ),
             model.Link(
-                title="TileMatrixSet",
+                title="TileMatrixSet (Template URL)",
                 href=self.url_for(
                     request,
                     "tilematrixset",
@@ -1142,9 +1186,12 @@ class OGCTilesFactory(EndpointsFactory):
                 ),
                 type=MediaType.json,
                 rel="data",
+                templated=True,
             ),
         ]
 
+        return links
+
     def register_routes(self):  # noqa: C901
         """Register OGC Tiles endpoints."""
         self._tilematrixsets_routes()
@@ -1201,8 +1248,9 @@ class OGCTilesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="tilematrixsets",
+                    title="TileMatrixSets list",
                 )
 
             return data
@@ -1234,23 +1282,21 @@ class OGCTilesFactory(EndpointsFactory):
             """
             OGC Specification: http://docs.opengeospatial.org/per/19-069.html#_tilematrixset
             """
-            data = self.supported_tms.get(tileMatrixSetId)
+            tms = self.supported_tms.get(tileMatrixSetId)
 
             if output_type == MediaType.html:
-                # For visualization purpose we add the tms bbox
-                data = {
-                    **data.model_dump(exclude_none=True, mode="json"),
-                    "bbox": data.bbox,
-                }
                 return self._create_html_response(
                     request,
-                    json.dumps(
-                        data,
-                    ),
+                    {
+                        **tms.model_dump(exclude_none=True, mode="json"),
+                        # For visualization purpose we add the tms bbox
+                        "bbox": tms.bbox,
+                    },
                     template_name="tilematrixset",
+                    title=f"{tileMatrixSetId} TileMatrixSet",
                 )
 
-            return data
+            return tms
 
     def _tilesets_routes(self):
         @self.router.get(
@@ -1338,8 +1384,9 @@ class OGCTilesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="tilesets",
+                    title=f"{collection.id} tilesets",
                 )
 
             return data
@@ -1402,49 +1449,67 @@ class OGCTilesFactory(EndpointsFactory):
                     for matrix in tms
                 ]
 
+            links = [
+                {
+                    "href": self.url_for(
+                        request,
+                        "collection_tileset",
+                        collectionId=collection.id,
+                        tileMatrixSetId=tileMatrixSetId,
+                    ),
+                    "rel": "self",
+                    "type": "application/json",
+                    "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
+                },
+                {
+                    "href": self.url_for(
+                        request,
+                        "tilematrixset",
+                        tileMatrixSetId=tileMatrixSetId,
+                    ),
+                    "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
+                    "type": "application/json",
+                    "title": f"Definition of '{tileMatrixSetId}' tileMatrixSet",
+                },
+                {
+                    "href": self.url_for(
+                        request,
+                        "collection_get_tile",
+                        tileMatrixSetId=tileMatrixSetId,
+                        collectionId=collection.id,
+                        z="{z}",
+                        x="{x}",
+                        y="{y}",
+                    ),
+                    "rel": "tile",
+                    "type": "application/vnd.mapbox-vector-tile",
+                    "title": "Templated link for retrieving Vector tiles",
+                    "templated": True,
+                },
+            ]
+
+            if self.with_viewer:
+                links.append(
+                    {
+                        "href": self.url_for(
+                            request,
+                            "viewer_endpoint",
+                            tileMatrixSetId=tileMatrixSetId,
+                            collectionId=collection.id,
+                        ),
+                        "type": "text/html",
+                        "rel": "data",
+                        "title": f"Map viewer for '{tileMatrixSetId}' tileMatrixSet",
+                    }
+                )
+
             data = model.TileSet.model_validate(
                 {
                     "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
                     "dataType": "vector",
                     "crs": tms.crs,
                     "boundingBox": collection_bbox,
-                    "links": [
-                        {
-                            "href": self.url_for(
-                                request,
-                                "collection_tileset",
-                                collectionId=collection.id,
-                                tileMatrixSetId=tileMatrixSetId,
-                            ),
-                            "rel": "self",
-                            "type": "application/json",
-                            "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
-                        },
-                        {
-                            "href": self.url_for(
-                                request,
-                                "tilematrixset",
-                                tileMatrixSetId=tileMatrixSetId,
-                            ),
-                            "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
-                            "type": "application/json",
-                            "title": f"Definition of '{tileMatrixSetId}' tileMatrixSet",
-                        },
-                        {
-                            "href": self.url_for(
-                                request,
-                                "collection_get_tile",
-                                tileMatrixSetId=tileMatrixSetId,
-                                collectionId=collection.id,
-                                z="{z}",
-                                x="{x}",
-                                y="{y}",
-                            ),
-                            "rel": "tile",
-                            "type": "application/vnd.mapbox-vector-tile",
-                            "title": "Templated link for retrieving Vector tiles",
-                        },
-                    ],
+                    "links": links,
                     "tileMatrixSetLimits": tilematrix_limit,
                 }
             )
@@ -1452,8 +1517,9 @@ class OGCTilesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.model_dump_json(exclude_none=True),
+                    data.model_dump(exclude_none=True, mode="json"),
                     template_name="tileset",
+                    title=f"{collection.id} {tileMatrixSetId} tileset",
                 )
 
             return data
@@ -1472,6 +1538,7 @@ class OGCTilesFactory(EndpointsFactory):
             responses={200: {"content": {MediaType.mvt.value: {}}}},
             operation_id=".collection.vector.getTile",
             tags=["OGC Tiles API"],
+            deprecated=True,
         )
         async def collection_get_tile(
             request: Request,
@@ -1555,6 +1622,7 @@ class OGCTilesFactory(EndpointsFactory):
             response_class=ORJSONResponse,
             operation_id=".collection.vector.getTileJSON",
             tags=["OGC Tiles API"],
+            deprecated=True,
         )
         async def collection_tilejson(
             request: Request,
@@ -1656,6 +1724,7 @@ class OGCTilesFactory(EndpointsFactory):
             response_class=ORJSONResponse,
             operation_id=".collection.vector.getStyleJSON",
             tags=["OGC Tiles API"],
+            deprecated=True,
         )
         async def collection_stylejson(
             request: Request,
@@ -1713,12 +1782,7 @@ class OGCTilesFactory(EndpointsFactory):
             minzoom = minzoom if minzoom is not None else tms.minzoom
             maxzoom = maxzoom if maxzoom is not None else tms.maxzoom
 
-            bounds = collection.bounds or [
-                180,
-                -85.05112877980659,
-                180,
-                85.0511287798066,
-            ]
+            bounds = list(collection.bounds) or list(tms.bbox)
 
             style_json = {
                 "name": "TiPg",
@@ -1787,19 +1851,29 @@ class OGCTilesFactory(EndpointsFactory):
                 "/collections/{collectionId}/{tileMatrixSetId}/viewer",
                 response_class=HTMLResponse,
                 operation_id=".collection.vector.viewerTms",
+                deprecated=True,
+                tags=["Map Viewer"],
             )
             @self.router.get(
                 "/collections/{collectionId}/viewer",
                 response_class=HTMLResponse,
                 operation_id=".collection.vector.viewer",
+                deprecated=True,
+                tags=["Map Viewer"],
+            )
+            @self.router.get(
+                "/collections/{collectionId}/tiles/{tileMatrixSetId}/viewer",
+                response_class=HTMLResponse,
+                operation_id=".collection.vector.map",
+                tags=["Map Viewer"],
             )
             def viewer_endpoint(
                 request: Request,
                 collection: Annotated[Collection, Depends(self.collection_dependency)],
                 tileMatrixSetId: Annotated[
-                    Literal["WebMercatorQuad"],
-                    "Identifier selecting one of the TileMatrixSetId supported (default: 'WebMercatorQuad')",
-                ] = "WebMercatorQuad",
+                    Literal[tuple(self.supported_tms.list())],
+                    f"Identifier selecting one of the TileMatrixSetId supported (default: '{tms_settings.default_tms}')",
+                ] = tms_settings.default_tms,
                 minzoom: Annotated[
                     Optional[int],
                     Query(description="Overwrite default minzoom."),
@@ -1817,7 +1891,7 @@ class OGCTilesFactory(EndpointsFactory):
                 ] = None,
             ):
                 """Return Simple HTML Viewer for a collection."""
-                self.supported_tms.get(tileMatrixSetId)
+                tms = self.supported_tms.get(tileMatrixSetId)
 
                 tilejson_url = self.url_for(
                     request,
@@ -1828,13 +1902,16 @@ class OGCTilesFactory(EndpointsFactory):
                 if request.query_params._list:
                     tilejson_url += f"?{urlencode(request.query_params._list)}"
 
-                return self.templates.TemplateResponse(
+                return self._create_html_response(
                     request,
-                    name="map.html",
-                    context={
+                    {
+                        "title": collection.id,
                         "tilejson_endpoint": tilejson_url,
+                        "tms": tms,
+                        "resolutions": [matrix.cellSize for matrix in tms],
                     },
-                    media_type="text/html",
+                    template_name="map",
+                    title=f"{collection.id} viewer",
                 )
 
 
